title = 'bigbucket'
description = 'Python3 client for Bigbucket'
url = 'https://github.com/adrianchifor/bigbucket-python'
version = '0.2.0'
author = 'Adrian Chifor'
author_email = 'me@adrianchifor.com'
license = 'GNU General Public License v3 (GPLv3)'
